const path = require('path');
const entryProvider = require('../utilities/entry-provider');
const {ProvidePlugin, IgnorePlugin} = require('webpack');
const DeadCodePlugin = require('webpack-deadcode-plugin');

const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const {WebSitesManifestPlugin} = require("../plugins/WebSitesManifestPlugin");
const {RemoveAssetsPlugin} = require("../plugins/RemoveAssetsPlugin");


const commonChunkName = 'webpack-common-chunk';


const cssLoader = {
    loader: 'css-loader',
    options: {
        importLoaders: 1,
        sourceMap: true
    }
};

const cssLoaderWithModules = {
    loader: 'css-loader',
    options: {
        modules: true,
        localIdentName: '[local]--[md5:contenthash:hex:8]',
        sourceMap: true
    }
};

const lessLoader = {
    loader: 'less-loader',
    options: {
        sourceMap: true
    }
};

const babelLoader = {
    loader: 'babel-loader',
    options: {
        presets: ["@babel/env", "@babel/react"],
        plugins: [
            ["@babel/proposal-decorators", {legacy: true}],
            ["@babel/proposal-class-properties", {loose: true}]
        ],
    }
};

const cssModuleRegex = /\.module\.css$/;

module.exports = (env, options) => {
    const isProduction = options.mode === 'production';
    return ({
        devtool: isProduction ? 'none' : 'cheap-module-eval-source-map',

        entry: entryProvider.getEntries('./front/entry/', '**/*.js'),
        output: {
            library: 'returnValue',
            libraryExport: 'default',
            globalObject: 'this',
            path: path.resolve('./front/build'),
            filename: '[name].[contenthash].js'
        },

        resolve: {
            modules: ['.', 'node_modules'],
            extensions: ['*', '.js', '.jsx']
        },
        module: {
            rules: [
                {
                    test: /\.(js|jsx)$/,
                    use: [babelLoader]
                },
                {
                    test: cssModuleRegex,
                    use: [MiniCssExtractPlugin.loader, cssLoaderWithModules]
                },
                {
                    test: /\.(css|less)$/,
                    exclude: cssModuleRegex,
                    sideEffects: true,
                    use: [MiniCssExtractPlugin.loader, cssLoader, lessLoader],

                },
                {
                    test: /\.(png|jpg|jpeg|gif|svg|eot|woff|woff2|ttf|swf)$/,
                    loader: 'file-loader',
                    options: {
                        name: 'assets/[name].[contenthash].[ext]',
                    },
                },
            ]
        },
        stats: {children: false},
        plugins: [
            !isProduction && new DeadCodePlugin({
                detectUnusedExport: false,
                patterns: [
                    'front/src/**/*.js',
                    'theme/**/*.js',
                ],
                exclude: [
                    '**/theme/layout/**',
                ],
            }),
            new ProvidePlugin({
                $: "jquery",
                jQuery: "jquery"
            }),
            new IgnorePlugin(/^\.\/locale$/, /moment$/),
            new MiniCssExtractPlugin({
                filename: '[name].[contenthash].css',
            }),
            new RemoveAssetsPlugin({
                shouldDelete: /css\\.*\.js(\.map)?$/
            }),
            new WebSitesManifestPlugin()
            ,
        ].filter(Boolean),
        optimization: {
            runtimeChunk: {
                name: commonChunkName
            },
            splitChunks: {
                cacheGroups: {
                    [commonChunkName]: {
                        name: commonChunkName,
                        chunks: 'initial',
                        minChunks: 2
                    }
                }
            },
            minimizer: [
                new TerserPlugin(),
                new OptimizeCssAssetsPlugin()
            ]
        }
    });
};
